; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version 10/2016																
; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)									
; Beispiel:																		
; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)									
TWI_INI:
	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	out TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	out TWCR,temp
	; PULL-UP ein								
	sbi PORTC,PC4
	sbi PORTC,PC5
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_SENSOR_CHECK:
	ldi temp1,(TWI_ADRESSE_HYT939)
	rcall TWI_MASTER_TRANSMITTER_START_TEMP1
	rcall TWI_STOP		
	tst TWI_FEHLER_CODE
	brne TWI_SENSOR_CHECK_HH10D
	STS(adr_HYT939_VORHANDEN),VOLL			; vorhanden !	
	ret
TWI_SENSOR_CHECK_HH10D:
	ldi temp1,(TWI_ADRESSE_HH10D)
	rcall TWI_MASTER_TRANSMITTER_START_TEMP1
	rcall TWI_STOP		
	tst TWI_FEHLER_CODE	
	brne TWI_SENSOR_CHECK__ERROR
	rcall HH10D_EEPROM_READ					; EEPROM		
	tst TWI_FEHLER_CODE	
	brne TWI_SENSOR_CHECK__ERROR
	STS(adr_HH10D_VORHANDEN),VOLL			; vorhanden !	
	ret
TWI_SENSOR_CHECK__ERROR:
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)	; ROT  an		
	ldi temp,(ERROR_NO_SENSOR)
	STS(adr_RH),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			   	
; START-ROUTINE	(MASTER TRANSMITTER)										   	
;																			   	
; INPUT:	temp1 <= ADRESSE SLAVE											   	
; OUT: 		FEHLER_CODE															
TWI_MASTER_TRANSMITTER_START_TEMP1:
	clr TWI_FEHLER_CODE
	WDR
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)	; START-MT						
	out TWCR, temp	
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1	
	; Check TWSR						
	in temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08								; 0x08 START erfolgreich		
	breq TWI_MASTER_TRANSMITTER_START_w
	cpi temp,  0x10								; 0x10 (repeated START)			
	breq TWI_MASTER_TRANSMITTER_START_w
	rjmp TWI_MASTER_TRANSMITTER_START_ERROR
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1									; Adresse 1 x links				
	out TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1	
	; Check TWSR						
	in temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18								; 0x18							
	brne TWI_MASTER_TRANSMITTER_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_ERROR:
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   	
; Ein BYTE zum SLAVE zu senden										   	   	   	
; 																			   	
; INPUT:	temp1 <= DATEN													   	
; OUT:		FEHLER_CODE															
; 																			   	
TWI_MASTER_TRANSMITTER_DATEN_TEMP1:
	WDR
	tst TWI_FEHLER_CODE
	breq TWI_MASTER_TRANSMITTER_DATEN_RUN
	ret	
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATEN_RUN:
	out TWDR, temp1							; 	Load DATA	
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	rcall TWI_TWINT_wait
	; Check TWSR						
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	brne TWI_MASTER_TRANSMITTER_DATEN_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATEN_ERROR:
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			   	
; TWI/I2C START-ROUTINE														   	
;																			   	
; 	INPUT:	temp1 <= ADRESSE SLAVE											   	
;	OUT: 	FEHLER_CODE		   													
;																			   	
TWI_MASTER_RECEIVER_START_TEMP1:
	clr TWI_FEHLER_CODE
	WDR
	; START-MR 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	out TWCR, temp	
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1						
	; Check TWSR 										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 							; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_RUN
	cpi temp,  0x10								; 0x10 (repeated START)	
	breq TWI_MASTER_RECEIVER_START_RUN
	rjmp TWI_MASTER_RECEIVER_ERROR
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_RUN:
	; Load SLA +"R" (R=1)								
	LSL  temp1					; 1 x links	
	ori temp1, 0b00000001 		; R=1		
	out TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 				;SLA+R gesendet; ACK empfangen
	brne TWI_MASTER_RECEIVER_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_ERROR:
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! BYTES empfangen werden sollen										
;																				
; 	OUT: 	temp1																
;	OUT: 	FEHLER_CODE		   													
;																				
TWI_MASTER_RECEIVER_DATA_TEMP1:
	WDR
	tst TWI_FEHLER_CODE
	breq TWI_MASTER_RECEIVER_BYTE_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_BYTE_RUN:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	rcall TWI_TWINT_wait; TWINT abfragen									
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_BYTE_ERROR
	in temp1, TWDR		; DATEN einlesen											
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_BYTE_ERROR:
	clr temp1
	mov TWI_FEHLER_CODE, EINS	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! BYTE empfangen werden soll, oder wenn								
; das letzte! BYTE empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RECEIVER_DATA_LAST_BYTE_TEMP1:
	; ERROR								
	tst TWI_FEHLER_CODE
	breq TWI_MASTER_RECEIVER_LAST_BYTE_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_LAST_BYTE_RUN:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	out TWCR, temp
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_LAST_BYTE_ERROR
	in temp1, TWDR	; DATEN einlesen		
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_LAST_BYTE_ERROR:
	clr temp1
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE
	out TWCR, temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	in temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
ret
;###############################################################################
;###############################################################################
;###############################################################################

