;														
; 1-WIRE-SLAVE 											
; ATmega8												
; Bernhard.Erfurt@gmx.de								
; 01/2017												
;														

;														
; CRC ROM-ID wird automatisch berechnet					
;														



;														
; GRN leuchtet: 1-WIRE									
; GELB leuchtet: Spannung messen						
; ROT  leuchtet: ERROR									
;														
; Blinkt die rote LED, das ist nicht gut !				
; ROT blinkt langsam:  	Fehler im Programmcheck			
; ROT blinkt schnell: 	unkorrekter Interruptaufruf		
;														

;														
; Anschlussbelegung PINS								
;														
; PB0 ==> LED-BALKEN									
; PB1 ==> 												
; PB2 ==> 												
; PB3 ==> 												
; PB4 ==> 												
; PB5 ==> 												
; PB6 ==> (Quarz)										
; PB7 ==> (Quarz)										
;														
; PC0 ==> ADC											
; PC1 ==> LED-BALKEN									
; PC2 ==> LED-BALKEN									
; PC3 ==> LED-BALKEN									
; PC4 ==> LED-BALKEN									
; PC5 ==> LED-BALKEN									
;														
; PD0 ==> LED-BALKEN									
; PD1 ==> LED-BALKEN									
; PD2 ==> 1-WIRE-BUS									
; PD3 ==> LED-BALKEN									
; PD4 ==> LED-BALKEN									
; PD5 ==> LED GRN										
; PD6 ==> LED GELB										
; PD7 ==> LED ROT										
;														


; ATmega8 											
.include "m8def.inc"
; KONSTANTEN										
.equ TAKT							=4000000			; 4MHz	
; TIMER-WERTE										
.equ TIMERWERT1						=65536-62500 ;;(2 hoch 16 minus....) bei 4MHz + Vorteiler 64 => 1s 		
; 1-WIRE-ROM-ID									
.equ ROM_ID_BYTE0					= 0xE0		;0x10 FAMILY xE0=PRIVAT (mit SCRATCHPAD CRC) xEF=PRIVAT
.equ ROM_ID_BYTE1					= 0x20
.equ ROM_ID_BYTE2					= 0x17
.equ ROM_ID_BYTE3					= 0x00
.equ ROM_ID_BYTE4					= 0x00
.equ ROM_ID_BYTE5					= 0x00
.equ ROM_ID_BYTE6					= 0x00
.equ ROM_ID_BYTE7					= 0x00		; CRC-DUMMY (wird automatisch korrigiert)			
; 1-WIRE-ROM-COMMANDS							
.equ OW_COMMAND_READ_ROM			= 0x33		; READ-ROM				
.equ OW_COMMAND_MATCH_ROM			= 0x55		; MATCH-ROM				
.equ OW_COMMAND_SEARCH_ROM			= 0xF0		; SEARCH-ROM			
.equ OW_COMMAND_SKIP_ROM			= 0xCC		; SKIP-ROM				
; 1-WIRE-FUNCTION-COMMANDS						
.equ OW_FUNCTION_CONVERT_T				=0x44		;CONVERT "T" DS19s20	
.equ OW_FUNCTION_MASTER_READ_SCRATCHPAD	=0xBE		; read  DS19s20			
.equ OW_FUNCTION_MASTER_WRITE_SCRATCHPAD=0x4E		; write DS19s20			
.equ OW_FUNCTION_MASTER_READ_DATA		=0xF0		; read  DATA (privat)	
.equ OW_FUNCTION_MASTER_WRITE_DATA		=0xF6		; write DATA (privat)	
; 1-WIRE-STATUS nicht ndern !!					
.equ OW_STATUS_RX_BEREIT			= 0b00000001	; Empfangsbereit								
.equ OW_STATUS_TX_0SLOT				= 0b00000010	; eine "0" muss zum MASTER gesendet werden		
.equ OW_STATUS_TX_1SLOT				= 0b00000100	; eine "1" muss zum MASTER gesendet werden		
.equ OW_STATUS_GESPERRT				= 0b11110000	; GESPERRT (auf RESET-PRESENCE ist wird gewartet
; 1-WIRE-BUS									
.equ OW_PIN_NR						= 2
.equ OW_PORT						= PORTD
.equ OW_DDR							= DDRD
.equ OW_PIN							= PIND
; LED GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRD
.equ LED_GRUEN_PORT					= PORTD
; LED GELB										
.equ LED_GELB_PIN_NR				= 6
.equ LED_GELB_DDR					= DDRD
.equ LED_GELB_PORT					= PORTD
; LED ROT										
.equ LED_ROT_PIN_NR					= 7
.equ LED_ROT_DDR					= DDRD
.equ LED_ROT_PORT					= PORTD
; LED 0											
.equ LED_0_PIN_NR					= 0
.equ LED_0_DDR						= DDRD
.equ LED_0_PORT						= PORTD
; LED 1											
.equ LED_1_PIN_NR					= 1
.equ LED_1_DDR						= DDRD
.equ LED_1_PORT						= PORTD
; LED 2											
.equ LED_2_PIN_NR					= 3
.equ LED_2_DDR						= DDRD
.equ LED_2_PORT						= PORTD
; LED 3											
.equ LED_3_PIN_NR					= 4
.equ LED_3_DDR						= DDRD
.equ LED_3_PORT						= PORTD
; LED 4											
.equ LED_4_PIN_NR					= 0
.equ LED_4_DDR						= DDRB
.equ LED_4_PORT						= PORTB
; LED 5											
.equ LED_5_PIN_NR					= 1
.equ LED_5_DDR						= DDRC
.equ LED_5_PORT						= PORTC
; LED 6											
.equ LED_6_PIN_NR					= 2
.equ LED_6_DDR						= DDRC
.equ LED_6_PORT						= PORTC
; LED 7											
.equ LED_7_PIN_NR					= 3
.equ LED_7_DDR						= DDRC
.equ LED_7_PORT						= PORTC
; LED 8											
.equ LED_8_PIN_NR					= 4
.equ LED_8_DDR						= DDRC
.equ LED_8_PORT						= PORTC
; LED 9											
.equ LED_9_PIN_NR					= 5
.equ LED_9_DDR						= DDRC
.equ LED_9_PORT						= PORTC


; REGISTER										

.def OW_INTERRUPT		= R5

.def OW_BYTE			= R6	; wird nur im OW-ISR genutzt						
.def OW_BIT_NR			= R7	; wird nur im OW-ISR genutzt						

.def SREG_KOPIE			= R8	; 
.def temp_KOPIE			= R9	; 

.def ZL_KOPIE			= R10	; 
.def ZH_KOPIE			= R11	; 
.def STATUS				= R12

.def NULL 				= R13
.def EINS 				= R14
.def VOLL				= R15

.def temp 				= R16
.def temp1	 			= R17
.def temp2 				= R18
.def temp3 				= R19
.def temp4 				= R20
.def temp5 				= R21
.def temp6 				= R22
.def temp7 				= R23
.def temp8 				= R24
.def temp9 				= R25

; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						=SRAM_START+0		; 0x0060 START Adresse 		

.equ adr_FREIGABE_MESSEN			=SRAM_START+1

.equ adr_ADC_0						=SRAM_START+50


.equ adr_OW_ZAEHLER_RESET_PRESENCE_L=SRAM_START+100	; fr Wartungszwecke (ZAEHLERWERTE)	
.equ adr_OW_ZAEHLER_RESET_PRESENCE_H=SRAM_START+101	; 									

.equ adr_OW_BIT_NR					=SRAM_START+110	; Zhler							
.equ adr_OW_BYTE_ZAEHLER			=SRAM_START+111	; Zhler							

.equ adr_OW_ROM_MATCH_ZAEHLER		=SRAM_START+112	; Zhler							

.equ adr_OW_ROM_SEARCH_ZAEHLER_AB	=SRAM_START+120	; Zhler							
.equ adr_OW_ROM_SEARCH_BIT_ZAEHLER	=SRAM_START+121	; Zhler							
.equ adr_OW_ROM_SEARCH_ID_BYTE_NR	=SRAM_START+122	;
.equ adr_OW_ROM_SEARCH_BYTE			=SRAM_START+123	;

.equ adr_OW_ROM_COMMAND				=SRAM_START+124	; ROM COMMAND 						
.equ adr_OW_ROM_COMMAND_GESPERRT	=SRAM_START+125	; ROM COMMAND abgearbeitet			
.equ adr_OW_FUNCTION_COMMAND		=SRAM_START+126	; FUNCTION COMMAND					
.equ adr_OW_FUNCTION_COMMAND_GESPERRT=SRAM_START+127	; FUNCTION COMMAND abgearbeitet		


.equ adr_OW_ROM_ID_BEREICH8			=SRAM_START+250	; (8 BYTE) ROM-ID SLAVE				

.equ adr_OW_SCRATCHPAD_BEREICH9		=SRAM_START+300	; (9 BYTE) SCRATCHPAD  				

.equ adr_OW_DATEN_BEREICH256		=SRAM_START+600	; (256 BYTE) DATEN  				

.equ adr_SRAM_CHECK					=SRAM_START+943	; SRAM CHECK muss immer "S" sein	



.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp EXT_INT0 		  	; IRQ0 Handler     										
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF     		; Timer1 Overflow Handler       
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF   		; Timer0 Overflow Handler       
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	
; Unkorrekter Interrupt Aufruf 													
INTERRUPT_ERROR:
	CLI										; INTERRUPTS sperren	
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)	; GRN aus	
	sbi (LED_GELB_PORT) ,(LED_GELB_PIN_NR)	; GELB aus	
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)	; ROT  an	
	rcall WAIT_100ms
	sbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)	; ROT  aus	
	rcall WAIT_100ms
	rjmp INTERRUPT_ERROR
RESET:                        
; BALKEN											
	sbi (LED_9_DDR),(LED_9_PIN_NR)			; 9
	cbi (LED_9_PORT),(LED_9_PIN_NR)			; 9 an	
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; WDR_INITIALISIERUNG								
	rcall WDR_INITIALISIERUNG
; NEUSTART_MIT_WDR									
	rcall NEUSTART_MIT_WDR
; SRAM lschen										
	rcall SRAM_CLEAR
; REGISTER_CLEAR									
	rcall REGISTER_CLEAR
; PORT_CLEAR										
	rcall PORT_CLEAR
; LED_INITIALISIERUNG								
	rcall LED_INITIALISIERUNG
; STARTWERTE										
	; NULL					
	clr NULL
	; EINS					
	ldi temp,1
	mov EINS,temp
	; VOLL					
	ldi temp,255
	mov VOLL,temp
	; SRAM CHECK			
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
; 1-WIRE_INITIALISIERUNG							
	rcall ONE_WIRE_INITIALISIERUNG
; 1-WIRE-EXTERNER_INTERUPT_INITIALISIEREN			
	rcall EXTERNER_INTERUPT_INITIALISIEREN
; 1-WIRE-EXTERNER_INTERRUPT_AKTIVIEREN				
	rcall EXTERNER_INTERRUPT_AKTIVIEREN	
; ADC_INITIALISIERUNG								
	rcall ADC_INITIALISIERUNG
; 	PROGRAMM-CHECK									
	rcall PROGRAMM_CHECK
; Interrupt															
	out GIFR,VOLL			; eventuelle Pegelwechsel am 1wire-PIN ignorieren	
	SEI						; interrupts freigeben					
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR										; Watch-Dog-RESET	
	rcall PROGRAMM_CHECK
	LDS temp,(adr_FREIGABE_MESSEN)
	tst temp
	breq LOOP
;-------------------------------------------------------------------------------
	STS(adr_FREIGABE_MESSEN),NULL
	rcall MESSUNG
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
MESSUNG:
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)	; GRN aus	
	cbi (LED_GELB_PORT),(LED_GELB_PIN_NR) 	; GELB an	
	clr OW_INTERRUPT
	rcall LED_BALKEN_AUS
	ldi XL,255
	ldi temp9,100
MESSUNG_s:
	rcall MESSEN_0_TEMP1
	rcall BERECHNUNG_MIN_TEMP1
	rcall WAIT_1ms
	rcall WAIT_1ms
	dec temp9
	brne MESSUNG_s
	rcall LED_BALKEN_TEMP1
	sbi (LED_GELB_PORT),(LED_GELB_PIN_NR) 	; GELB aus	
	; ERROR ?		
	tst OW_INTERRUPT
	brne MESSUNG_ERROR
	ret
;-------------------------------------------------------------------------------
MESSUNG_ERROR:
	cbi (LED_ROT_PORT),(LED_ROT_PIN_NR)		; ROT an	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
BERECHNUNG_MIN_TEMP1:
	cp XL,temp1
	brsh BERECHNUNG_MIN_TEMP1_w
	mov temp1,XL
	ret
BERECHNUNG_MIN_TEMP1_w:
	mov XL,temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_BALKEN_TEMP1:
	cpi temp1,(4000*255/5000)
	brsh LED_BALKEN_0
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_0:
	cpi temp1,(4100*255/5000)
	brsh LED_BALKEN_1
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_1:
	cpi temp1,(4200*255/5000)
	brsh LED_BALKEN_2
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_2:
	cpi temp1,(4300*255/5000)
	brsh LED_BALKEN_3
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	cbi (LED_2_PORT),(LED_2_PIN_NR)			; 2 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_3:	
	cpi temp1,(4400*255/5000)
	brsh LED_BALKEN_4
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	cbi (LED_2_PORT),(LED_2_PIN_NR)			; 2 an	
	cbi (LED_3_PORT),(LED_3_PIN_NR)			; 3 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_4:	
	cpi temp1,(4500*255/5000)
	brsh LED_BALKEN_5
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	cbi (LED_2_PORT),(LED_2_PIN_NR)			; 2 an	
	cbi (LED_3_PORT),(LED_3_PIN_NR)			; 3 an	
	cbi (LED_4_PORT),(LED_4_PIN_NR)			; 4 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_5:	
	cpi temp1,(4600*255/5000)
	brsh LED_BALKEN_6
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	cbi (LED_2_PORT),(LED_2_PIN_NR)			; 2 an	
	cbi (LED_3_PORT),(LED_3_PIN_NR)			; 3 an	
	cbi (LED_4_PORT),(LED_4_PIN_NR)			; 4 an	
	cbi (LED_5_PORT),(LED_5_PIN_NR)			; 5 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_6:	
	cpi temp1,(4700*255/5000)
	brsh LED_BALKEN_7
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	cbi (LED_2_PORT),(LED_2_PIN_NR)			; 2 an	
	cbi (LED_3_PORT),(LED_3_PIN_NR)			; 3 an	
	cbi (LED_4_PORT),(LED_4_PIN_NR)			; 4 an	
	cbi (LED_5_PORT),(LED_5_PIN_NR)			; 5 an	
	cbi (LED_6_PORT),(LED_6_PIN_NR)			; 6 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_7:	
	cpi temp1,(4800*255/5000)
	brsh LED_BALKEN_8
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	cbi (LED_2_PORT),(LED_2_PIN_NR)			; 2 an	
	cbi (LED_3_PORT),(LED_3_PIN_NR)			; 3 an	
	cbi (LED_4_PORT),(LED_4_PIN_NR)			; 4 an	
	cbi (LED_5_PORT),(LED_5_PIN_NR)			; 5 an	
	cbi (LED_6_PORT),(LED_6_PIN_NR)			; 6 an	
	cbi (LED_7_PORT),(LED_7_PIN_NR)			; 7 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_8:	
	cpi temp1,(4900*255/5000)
	brsh LED_BALKEN_9
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	cbi (LED_2_PORT),(LED_2_PIN_NR)			; 2 an	
	cbi (LED_3_PORT),(LED_3_PIN_NR)			; 3 an	
	cbi (LED_4_PORT),(LED_4_PIN_NR)			; 4 an	
	cbi (LED_5_PORT),(LED_5_PIN_NR)			; 5 an	
	cbi (LED_6_PORT),(LED_6_PIN_NR)			; 6 an	
	cbi (LED_7_PORT),(LED_7_PIN_NR)			; 7 an	
	cbi (LED_8_PORT),(LED_8_PIN_NR)			; 8 an	
	ret
;-------------------------------------------------------------------------------
LED_BALKEN_9:
	cbi (LED_0_PORT),(LED_0_PIN_NR)			; 0 an	
	cbi (LED_1_PORT),(LED_1_PIN_NR)			; 1 an	
	cbi (LED_2_PORT),(LED_2_PIN_NR)			; 2 an	
	cbi (LED_3_PORT),(LED_3_PIN_NR)			; 3 an	
	cbi (LED_4_PORT),(LED_4_PIN_NR)			; 4 an	
	cbi (LED_5_PORT),(LED_5_PIN_NR)			; 5 an	
	cbi (LED_6_PORT),(LED_6_PIN_NR)			; 6 an	
	cbi (LED_7_PORT),(LED_7_PIN_NR)			; 7 an	
	cbi (LED_8_PORT),(LED_8_PIN_NR)			; 8 an	
	cbi (LED_9_PORT),(LED_9_PIN_NR)			; 9 an	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "TE_1WIRE.asm"						; 1-wire-INITIALISIERUNG			
.include "TE_1WIRE_INTERRUPT.asm"			; 1-wire-INTERRUPTS					
.include "TE_1WIRE_ROM_COMMAND.asm"			; 1-wire ROM-COMMANDS				
.include "TE_1WIRE_FUNCTION_COMMAND.asm" 	; 1-wire FUNKTION-COMMANDS			
.include "TE_SONSTIGES.asm"					; sonstiges							
.include "TE_ADC.asm"						; ADC								
.include "TE_EXT_INTERRUPT.asm"				; ext. INTERRUPT					
.include "TE_WAIT_04MHZ.asm"				; 4 MHz								
; ##############################################################################
; ##############################################################################
; ##############################################################################

